// Test3_AecPCIeTC.cpp
//
// This console application demonstrates how to open a handle
// to a device in your driver, and communicate with the driver
// using Read, Write, and DeviceIoControl calls, as appropriate.
//
// This test program attempts to open the device with link name
// AecPCIe, which corresponds to Unit 0 of the
// AecPCIe device class in the driver.

//
// You can build this application using the command line compiler with
// the following command:
//
//		cl Test3_AecPCIeTC.cpp
//


#include <stdlib.h>
#include <stdio.h>
#include <windows.h>

#include <winioctl.h>
#include "..\AECioctl.h"


DWORD dwMSec;

typedef void VOIDFUNC();

// Prototypes
void Usage(void);
void ShowIoctlValues(void);

//	TODO:
//		You can redefine the IOCTL handler prototypes as needed, adding
//		appropriate parameters that can be collected from the command line.
//		To do this you must modify the command line parsing logic.  An
//		example of this is shown in comments throughout the test application.
//
//=== Parameterized IOCTL Example ===
// void Test_IOCTL_PARAMETERIZED(int nVal, ULONG dwVal);
void Test_IOCTL_AECPCI_PROCESS_BUFFER(void);
void Test_IOCTL_AEC_READ_PORT_UCHAR(void);
void Test_IOCTL_AEC_READ_PORT_USHORT(void);
void Test_IOCTL_AEC_READ_PORT_ULONG(void);
void Test_IOCTL_AEC_WRITE_PORT_UCHAR(void);
void Test_IOCTL_AEC_WRITE_PORT_USHORT(void);
void Test_IOCTL_AEC_WRITE_PORT_ULONG(void);

void AecGo(ULONG act, ULONG port, ULONG data);

void CloseIfOpen(void);

// Global data

#define N_IOCODES	7

// Names of IOCTL codes
//
char *IOnames[N_IOCODES+1] =
{

//=== Parameterized IOCTL Example ===
//	"IOCTL_PARAMETERIZED",
	"IOCTL_AECPCI_PROCESS_BUFFER",
	"IOCTL_AEC_READ_PORT_UCHAR",
	"IOCTL_AEC_READ_PORT_USHORT",
	"IOCTL_AEC_READ_PORT_ULONG",
	"IOCTL_AEC_WRITE_PORT_UCHAR",
	"IOCTL_AEC_WRITE_PORT_USHORT",
	"IOCTL_AEC_WRITE_PORT_ULONG",
	""
};

// IOCTL codes
//
int IOcodes[N_IOCODES+1] =
{

//=== Parameterized IOCTL Example ===
//	IOCTL_PARAMETERIZED,
	IOCTL_AECPCI_PROCESS_BUFFER,
	IOCTL_AEC_READ_PORT_UCHAR,
	IOCTL_AEC_READ_PORT_USHORT,
	IOCTL_AEC_READ_PORT_ULONG,
	IOCTL_AEC_WRITE_PORT_UCHAR,
	IOCTL_AEC_WRITE_PORT_USHORT,
	IOCTL_AEC_WRITE_PORT_ULONG,
	0
};


// Handle to device opened in driver.
//
HANDLE	hDevice = INVALID_HANDLE_VALUE;

char *sLinkName = "\\\\.\\AecPCIe0";	//'AEC' device zero

////////////////////////////////////////////////////////////////////////
// OpenByName
//
//		Open a handle to the requested device
//
HANDLE OpenByName(void)
{
	// Create a handle to the driver
	return CreateFile(sLinkName,
					  GENERIC_READ | GENERIC_WRITE,
					  FILE_SHARE_READ,
					  NULL,
					  OPEN_EXISTING,
					  0,
					  NULL);
}

////////////////////////////////////////////////////////////////////////
// Exit
//
//		Print a message and exit
//
   void Exit(int res)
{
	printf("Exiting...\n\n");
	CloseIfOpen();
	exit(res);
}


////////////////////////////////////////////////////////////////////////
// Main entry point
//
//
int __cdecl main(int argc, char *argv[])
{
	int		nArgIndex;				// Walk through command line arguments
	int		nArgIncrement = 0;
	int		val;


	printf("Test application Test3_AecPCIeTC starting...\n");
	dwMSec = 1000;

	hDevice = OpenByName();
	if (hDevice == INVALID_HANDLE_VALUE)
	{
		printf("ERROR opening device: (%0x) returned from CreateFile\n", GetLastError());
		Sleep(4000);
		Exit(1);
	}
	else
	{
		printf("Device found, handle open.\n");
		Sleep(2000);
	}

	// Parse the command line

	if (argc < 2) Usage();

	nArgIndex = 1;
	while (nArgIndex < argc)
	{
		// Parse ahead to determine numeric value of argument

		if (nArgIndex+1 >= argc) Usage();
		if (!isdigit(argv[nArgIndex+1][0])) Usage();
		val = atoi(argv[nArgIndex+1]);

		switch (argv[nArgIndex][0])
		{

			case 'r':
			case 'R':
				printf("Driver does not have a read handler\n");
				Sleep(2000);
				nArgIncrement = 2;
				break;

			case 'w':
			case 'W':
				printf("Driver does not have a write handler\n");
				Sleep(2000);
				nArgIncrement = 2;
				break;

			case 'i':
			case 'I':
				if (val >= N_IOCODES)
				{
					printf("IO control code index must be less than %d\n", N_IOCODES);
					ShowIoctlValues();
					Sleep(6000);
					Exit(1);
				}
				switch (IOcodes[val])
				{

//=== Parameterized IOCTL Example ===
//					case IOCTL_PARAMETERIZED:
//						if (nArgIndex+3 >= argc) Usage();
//						nVal = atoi(argv[nArgIndex+2]);
//						dwVal = strtoul(argv[nArgIndex+3], NULL, 0);
//						Test_IOCTL_PARAMETERIZED(nVal, dwVal);
//						nArgIncrement = 4;
//						break;

					//This is the ONLY case currently supported by the PCIe-TC driver
					case IOCTL_AECPCI_PROCESS_BUFFER:
						Test_IOCTL_AECPCI_PROCESS_BUFFER();
						nArgIncrement = 2;
						break;

					case IOCTL_AEC_READ_PORT_UCHAR:
//						Test_IOCTL_AEC_READ_PORT_UCHAR();
						nArgIncrement = 2;
						break;

					case IOCTL_AEC_READ_PORT_USHORT:
//						Test_IOCTL_AEC_READ_PORT_USHORT();
						nArgIncrement = 2;
						break;

					case IOCTL_AEC_READ_PORT_ULONG:
//						Test_IOCTL_AEC_READ_PORT_ULONG();
						nArgIncrement = 2;
						break;

					case IOCTL_AEC_WRITE_PORT_UCHAR:
//						Test_IOCTL_AEC_WRITE_PORT_UCHAR();
						nArgIncrement = 2;
						break;

					case IOCTL_AEC_WRITE_PORT_USHORT:
//						Test_IOCTL_AEC_WRITE_PORT_USHORT();
						nArgIncrement = 2;
						break;

					case IOCTL_AEC_WRITE_PORT_ULONG:
//						Test_IOCTL_AEC_WRITE_PORT_ULONG();
						nArgIncrement = 2;
						break;

					default:
						printf("IO control code not valid\n");
						Sleep(2000);
						Exit(1);

				}
				break;

			case '?':
			case 'h':
			default:
				Usage();
		}
		nArgIndex += nArgIncrement;
	}

	return 0;
}


////////////////////////////////////////////////////////////////////////
// CloseIfOpen
//
//		Close the device if we previously opened a handle to it.
//
void CloseIfOpen(void)
{
	if (hDevice != INVALID_HANDLE_VALUE)
	{
		// Close the handle to the driver
		if (!CloseHandle(hDevice))
		{
			printf("ERROR: CloseHandle returns %0x.\n", GetLastError());
		}
		hDevice = INVALID_HANDLE_VALUE;
	}
}

////////////////////////////////////////////////////////////////////////
// Usage
//
//		Print a usage message describing arguments to this program
//
void Usage(void)
{
	printf("Usage: Test3_AecPCIeTC [r n] [w n] [i n]\n");
	printf("       i initiates an IO Control Code message with specified index value\n");
	ShowIoctlValues();
	printf("Example:\n");
	printf("    Test3_AecPCIeTC r 32 w 32\n");
	printf("        read 32 bytes, then write 32 bytes\n");

	Exit(1);
}

#define	IOCTL_INBUF_SIZE	512
#define	IOCTL_OUTBUF_SIZE	512

//=== Parameterized IOCTL Example ===
//void Test_IOCTL_PARAMETERIZED(int nVal, ULONG dwVal)
//{
//  Function body same as other IOCTL handlers, with command line
//	parameters 'nVal' and 'dwVal' available as input.
//}

////////////////////////////////////////////////////////////////////////
// Test_IOCTL_AECPCI_PROCESS_BUFFER
//
//		Test one Io Control Code
//
// TODO:
//		Pass appropriate arguments to your device and check
//		the return value
//
void Test_IOCTL_AECPCI_PROCESS_BUFFER(void)
{
// Note that Input and Output are named from the point of view
// of the DEVICE:
//		bufInput  supplies data to the device
//		bufOutput is written by the device to return data to this application

	UCHAR local_B0;
	ULONG local_R4;
	int loopCnt = 0;
	dwMSec = 350;

	loopCnt = 0;	//init
	printf("Start R1 at 00h \n");
	do	//a "read of AEC FPGA register 00h ('AEh')
	{
		//local_B0 = SubAecR1(0x00);
		AecGo(Read1, 0x00, 0x00); //
		printf("\n");
		loopCnt++;

	} while (loopCnt < 3);
	printf("End R1 at 00h \n");

	AecGo(Write1, 0x2E, 0x00); //SubAecW1(0x2E, 0x00);	//diable "int enables)
	printf("\n");
	AecGo(Write1, 0xFC, 0x00); //SubAecW1(0xFC, 0x00);	//Clr IRQ ability
	printf("\n");

	AecGo(Read1, 0x0F, 0x00); //
	printf("\n");
	AecGo(Write1, 0x2F, 0x00); //SubAecW1(0x2F, 0x00);		//Enable 'next'
	printf("\n");

	AecGo(Read1, 0xFE, 0x00); //
	printf("\n");
		
	loopCnt = 0;
	dwMSec = 10;	//msec
	AecGo(Read4, 0x10, 0x00);
	AecGo(Read4, 0x14, 0x00);
	printf("TnU \n");
	do
	{
		AecGo(Read4, 0x10, 0x00);
		printf(" cnt %d \r", loopCnt);
		loopCnt++;
		Sleep(25);

	} while (loopCnt < 2000);

	printf("\n");
	AecGo(Read4, 0x00, 0x00);
	printf("\n");
	AecGo(Read4, 0x04, 0x00);
	printf("\n");
	AecGo(Read4, 0x08, 0x00);
	printf("\n");
	AecGo(Read4, 0x0C, 0x00);
	printf("\n");
	AecGo(Read4, 0x10, 0x00);
	printf("\n");
	AecGo(Read4, 0x14, 0x00);
	printf("\n");
	AecGo(Read4, 0x18, 0x00);
	printf("\n");
	AecGo(Read4, 0x1C, 0x00);
	printf("\n");

	//===========
	loopCnt = 0;
	dwMSec = 10;	//msec
	AecGo(Read4, 0x10, 0x00);
	AecGo(Read4, 0x14, 0x00);
	printf("\n");

	AecGo(Read1, 0xFE, 0x00);	//read "flags"
	printf("\n");

	AecGo(Write1, 0x2F, 0x20); //"idle"command
	printf("\n");
	AecGo(Read1, 0xFE, 0x00);	//read "flags"
	printf("\n");

	AecGo(Read1, 0x0F, 0x00);	//read command
	printf("\n");
	AecGo(Read1, 0xFE, 0x00);	//read "flags"
	printf("\n");

	AecGo(Write1, 0x2F, 0x00);
	printf("\n");

	loopCnt = 0;
	printf("\n");
	do
	{
		AecGo(RTM, 0x00, 0x00);
		printf("cnt = %d \r", loopCnt);
		loopCnt++;
		Sleep(200);

	} while (loopCnt < 75);

	printf("\n");
	printf(". \n");
	AecGo(Write1, 0x2E, 0x7F);
	printf("\n");

	AecGo(Read1, 0xFE, 0x00);	//read "flags"
	printf("\n");

	AecGo(Write1, 0x2F, 0x2F);	//CMD
	printf("\n");

	AecGo(Read1, 0xFE, 0x00);	//read "flags"
	printf("\n");

	AecGo(Read1, 0x0F, 0x00);
	printf("\n");

	AecGo(Read1, 0xFE, 0x00);	//read "flags"
	printf("\n");

	AecGo(Write1, 0x2F, 0x00);
	printf("\n");

	Sleep (100);
	AecGo(RTM, 0x00, 0x00);
	printf("\n");
	AecGo(RUB, 0x01, 0x00);
	printf("\n");
	AecGo(REB, 0x02, 0x00);
	printf("\n");

	do
	{
		AecGo(RTM, 0x00, 0x00);
		printf(" %d\r", loopCnt);
		loopCnt++;
		Sleep(20);

	} while (loopCnt < 200);

	printf("..\n");
	AecGo(SND_CMD, 0x2F, 0x00);
	Sleep(40);

	AecGo(GET_INT, 0x00, 0x00);
	printf("\n");

		//Do some write/read tests to R/W DPRAM registers
	AecGo(Write4, 0x20, 0x12594812);
	printf("\n");
	AecGo(Read4, 0x20, 0x00);
	printf("\n");

	AecGo(Write4, 0x20, 0xFEDBCA90);
	printf("\n");
	AecGo(Read4, 0x20, 0x00);
	printf("\n");

	AecGo(Write4, 0x20, 0x12345678);
	printf("\n");
	AecGo(Read4, 0x20, 0x00);
	printf("\n");


}

//////////////////////////////////////////////////////
void AecGo(ULONG act, ULONG port, ULONG data)
{
	BOOL status;
	AECPCI_ACTION_ITEM		ai;			// *PAECPCI_ACTION_ITEM;

	DWORD nOutput;

	ai.action = (AECPCI_ACTION)act;
	ai.port = port;
	ai.value = data;

	if (status = DeviceIoControl(		
                hDevice,			//1 Handle to device
                IOCTL_AECPCI_PROCESS_BUFFER,//2 IO Control code -- (Read)/ (Write)
								//pointer to buffer to supply input data
				&ai,			//3++ Buffer to driver.  Holds port & data (if any).
								//size of input buffer
				sizeof(ai),		//4+++ Length of buffer in bytes.
								//pointer to buffer to receive output data
                &ai,			//5++++ Buffer from driver.
								//size of output buffer
                sizeof(ai),		//6+++++ Length of buffer in bytes. 1/2/4 (char/word/dword)
								//pointer to variable to receive output byte count
				&nOutput,			//7 Bytes placed in DataBufferOut. (store # upon return)
                NULL			//8 NULL means wait till op. completes.
                )				
		)
	{		
		printf("Action = %d, port = %0X, data = %8.8X   ", ai.action, ai.port, ai.value);
	}
	else
	{
		printf("ERROR: DeviceIoControl returns %0x.", GetLastError());
		Exit(1);
	}


}

////////////////////////////////////////////////////////////////////////
// Test_IOCTL_AEC_READ_PORT_UCHAR
//
//		Test one Io Control Code
//
// TODO:
//		Pass appropriate arguments to your device and check
//		the return value
//
void Test_IOCTL_AEC_READ_PORT_UCHAR(void)
{
// Note that Input and Output are named from the point of view
// of the DEVICE:
//		bufInput  supplies data to the device
//		bufOutput is written by the device to return data to this application

	CHAR	bufInput[IOCTL_INBUF_SIZE];		// Input to device
	CHAR	bufOutput[IOCTL_OUTBUF_SIZE];	// Output from device
	ULONG	nOutput;						// Count written to bufOutput

	// Call device IO Control interface (IOCTL_AEC_READ_PORT_UCHAR) in driver
	printf("Issuing Ioctl to device - ");
	if (!DeviceIoControl(hDevice,
						 IOCTL_AEC_READ_PORT_UCHAR,
						 bufInput,
						 IOCTL_INBUF_SIZE,
						 bufOutput,
						 IOCTL_OUTBUF_SIZE,
						 &nOutput,
						 NULL)
	   )
	{
		printf("ERROR: DeviceIoControl returns %0x.", GetLastError());
		Exit(1);
	}
}

////////////////////////////////////////////////////////////////////////
// Test_IOCTL_AEC_READ_PORT_USHORT
//
//		Test one Io Control Code
//
// TODO:
//		Pass appropriate arguments to your device and check
//		the return value
//
void Test_IOCTL_AEC_READ_PORT_USHORT(void)
{
// Note that Input and Output are named from the point of view
// of the DEVICE:
//		bufInput  supplies data to the device
//		bufOutput is written by the device to return data to this application

	CHAR	bufInput[IOCTL_INBUF_SIZE];		// Input to device
	CHAR	bufOutput[IOCTL_OUTBUF_SIZE];	// Output from device
	ULONG	nOutput;						// Count written to bufOutput

	// Call device IO Control interface (IOCTL_AEC_READ_PORT_USHORT) in driver
	printf("Issuing Ioctl to device - ");
	if (!DeviceIoControl(hDevice,
						 IOCTL_AEC_READ_PORT_USHORT,
						 bufInput,
						 IOCTL_INBUF_SIZE,
						 bufOutput,
						 IOCTL_OUTBUF_SIZE,
						 &nOutput,
						 NULL)
	   )
	{
		printf("ERROR: DeviceIoControl returns %0x.", GetLastError());
		Exit(1);
	}
}

////////////////////////////////////////////////////////////////////////
// Test_IOCTL_AEC_READ_PORT_ULONG
//
//		Test one Io Control Code
//
// TODO:
//		Pass appropriate arguments to your device and check
//		the return value
//
void Test_IOCTL_AEC_READ_PORT_ULONG(void)
{
// Note that Input and Output are named from the point of view
// of the DEVICE:
//		bufInput  supplies data to the device
//		bufOutput is written by the device to return data to this application

	CHAR	bufInput[IOCTL_INBUF_SIZE];		// Input to device
	CHAR	bufOutput[IOCTL_OUTBUF_SIZE];	// Output from device
	ULONG	nOutput;						// Count written to bufOutput

	// Call device IO Control interface (IOCTL_AEC_READ_PORT_ULONG) in driver
	printf("Issuing Ioctl to device - ");
	if (!DeviceIoControl(hDevice,
						 IOCTL_AEC_READ_PORT_ULONG,
						 bufInput,
						 IOCTL_INBUF_SIZE,
						 bufOutput,
						 IOCTL_OUTBUF_SIZE,
						 &nOutput,
						 NULL)
	   )
	{
		printf("ERROR: DeviceIoControl returns %0x.", GetLastError());
		Exit(1);
	}
}

////////////////////////////////////////////////////////////////////////
// Test_IOCTL_AEC_WRITE_PORT_UCHAR
//
//		Test one Io Control Code
//
// TODO:
//		Pass appropriate arguments to your device and check
//		the return value
//
void Test_IOCTL_AEC_WRITE_PORT_UCHAR(void)
{
// Note that Input and Output are named from the point of view
// of the DEVICE:
//		bufInput  supplies data to the device
//		bufOutput is written by the device to return data to this application

	CHAR	bufInput[IOCTL_INBUF_SIZE];		// Input to device
	CHAR	bufOutput[IOCTL_OUTBUF_SIZE];	// Output from device
	ULONG	nOutput;						// Count written to bufOutput

	// Call device IO Control interface (IOCTL_AEC_WRITE_PORT_UCHAR) in driver
	printf("Issuing Ioctl to device - ");
	if (!DeviceIoControl(hDevice,
						 IOCTL_AEC_WRITE_PORT_UCHAR,
						 bufInput,
						 IOCTL_INBUF_SIZE,
						 bufOutput,
						 IOCTL_OUTBUF_SIZE,
						 &nOutput,
						 NULL)
	   )
	{
		printf("ERROR: DeviceIoControl returns %0x.", GetLastError());
		Exit(1);
	}
}

////////////////////////////////////////////////////////////////////////
// Test_IOCTL_AEC_WRITE_PORT_USHORT
//
//		Test one Io Control Code
//
// TODO:
//		Pass appropriate arguments to your device and check
//		the return value
//
void Test_IOCTL_AEC_WRITE_PORT_USHORT(void)
{
// Note that Input and Output are named from the point of view
// of the DEVICE:
//		bufInput  supplies data to the device
//		bufOutput is written by the device to return data to this application

	CHAR	bufInput[IOCTL_INBUF_SIZE];		// Input to device
	CHAR	bufOutput[IOCTL_OUTBUF_SIZE];	// Output from device
	ULONG	nOutput;						// Count written to bufOutput

	// Call device IO Control interface (IOCTL_AEC_WRITE_PORT_USHORT) in driver
	printf("Issuing Ioctl to device - ");
	if (!DeviceIoControl(hDevice,
						 IOCTL_AEC_WRITE_PORT_USHORT,
						 bufInput,
						 IOCTL_INBUF_SIZE,
						 bufOutput,
						 IOCTL_OUTBUF_SIZE,
						 &nOutput,
						 NULL)
	   )
	{
		printf("ERROR: DeviceIoControl returns %0x.", GetLastError());
		Exit(1);
	}
}

////////////////////////////////////////////////////////////////////////
// Test_IOCTL_AEC_WRITE_PORT_ULONG
//
//		Test one Io Control Code
//
// TODO:
//		Pass appropriate arguments to your device and check
//		the return value
//
void Test_IOCTL_AEC_WRITE_PORT_ULONG(void)
{
// Note that Input and Output are named from the point of view
// of the DEVICE:
//		bufInput  supplies data to the device
//		bufOutput is written by the device to return data to this application

	CHAR	bufInput[IOCTL_INBUF_SIZE];		// Input to device
	CHAR	bufOutput[IOCTL_OUTBUF_SIZE];	// Output from device
	ULONG	nOutput;						// Count written to bufOutput

	// Call device IO Control interface (IOCTL_AEC_WRITE_PORT_ULONG) in driver
	printf("Issuing Ioctl to device - ");
	if (!DeviceIoControl(hDevice,
						 IOCTL_AEC_WRITE_PORT_ULONG,
						 bufInput,
						 IOCTL_INBUF_SIZE,
						 bufOutput,
						 IOCTL_OUTBUF_SIZE,
						 &nOutput,
						 NULL)
	   )
	{
		printf("ERROR: DeviceIoControl returns %0x.", GetLastError());
		Exit(1);
	}
}


////////////////////////////////////////////////////////////////////////
// ShowIoctlValues
//
//		Print list of IO Control Code values for usage display
//
void ShowIoctlValues(void)
{
	int i;

	for (i=0; i<N_IOCODES; i++)
	{
		if (i==0)
			printf( "         IO control code index\n");
		printf( "           %d is code %s [%x]\n", i, IOnames[i], IOcodes[i]);
	}
}
